/* 
   Miscellaneous functions
   --------------------------------------------------------------------
   Linberto

   Copyright (C) 1999-2001 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

void exit_game( int status)
{
 sound_shutdown();
 if( opt.cfg_joystick == LIN_JOYSTICK_ON) 
     joystick_close( 0);
 keyboard_close();
 vga_setmode( TEXT);
 nls_exit();
 exit( status);
}

void *lin_calloc( int size)
{
 void *ptr;

 if( (ptr = calloc( 1, size + 1)) == NULL) {
      fprintf( stderr, _("\nNot enough memory.\n\n"));
      exit_game( 1);
 }

 return ptr;
}

void press_enter_str( void)
{
 int x, y, len;

 len = 8 * strlen( _(" PRESS ENTER TO CONTINUE ")) + 8;
 x = ( 320 - len) / 2;
 y = 190;

 gl_write( x, y, _(" PRESS ENTER TO CONTINUE "));
}

/*
 * Calculate the smaller X, Y and the greather width & height
 */
void misc_calculate_minmax( struct actor_struct *act, int *x, int *y, int *w, int *h)
{
 *x = MIN( act->sp->x, act->lp.lx);
 *y = MIN( act->sp->y, act->lp.ly);

 *w =    MAX( act->sp->width , act->lp.lw)
     + ( MAX( act->sp->x     , act->lp.lx)
     -   MIN( act->sp->x     , act->lp.lx));
 *h =    MAX( act->sp->height, act->lp.lh)
     + ( MAX( act->sp->y     , act->lp.ly)
     -   MIN( act->sp->y     , act->lp.ly));
}

void misc_save_last_position( struct actor_struct *act)
{
 /* Save the currect position in the last position structure */
 act->lp.lx = act->sp->x;
 act->lp.ly = act->sp->y;
 act->lp.lw = act->sp->width;
 act->lp.lh = act->sp->height;
}

void check_if_file_exist( char *filename)
{
 int fd;

 if( access( filename, F_OK) == -1) {
     fd = creat( filename, 0600);

     /* If the file can't be created then
        return silently, and use the default file */

     close( fd);
 }
}

FILE *linberto_openfile( char *name1, char *name2, char *mode)
{
 static FILE *fp;

 if( ( fp = fopen( name1, mode)) == NULL) {
       fprintf( stderr, _("\nCan't open file: %s: %s\n\nThe game is not installed. Trying default file...\n\n"), name1, strerror( errno));
       if( ( fp = fopen( name2, mode)) == NULL) {
             fprintf( stderr, _("\nCan't open default file: %s: %s\n\n"), name2, strerror( errno));
             exit_game( 1);
       }
       openedfile = name2;
       return fp;
 }

 openedfile = name1;
 return fp;
}

int menu( int x, int y, int len, int pos, int maxpos, int quitpos)
{
 draw_bar( x, y, len, pos, LIN_RED);

 while( 1) {
        if( keyboard_update() == 0 && joy_moved() == 0)
            continue;

        switch( lin_scancode) {
              case SCANCODE_ENTER:
              case SCANCODE_KEYPADENTER:
                     return pos;
              case SCANCODE_Q:
              case SCANCODE_ESCAPE:
                     return quitpos;
              case SCANCODE_CURSORLEFT:
              case SCANCODE_CURSORBLOCKLEFT:
              case SCANCODE_CURSORUP:
              case SCANCODE_CURSORBLOCKUP:
                     sound_send( SND_MENU);
                     pos = loopbar( --pos, maxpos, x, y, len);
                     continue;
              case SCANCODE_CURSORRIGHT:
              case SCANCODE_CURSORBLOCKRIGHT:
              case SCANCODE_CURSORDOWN:
              case SCANCODE_CURSORBLOCKDOWN:
                     sound_send( SND_MENU);
                     pos = loopbar( ++pos, maxpos, x, y, len);
                     continue;
              case SCANCODE_HOME:
              case SCANCODE_KEYPAD7:
                     sound_send( SND_MENU);
                     pos = loopbar( 0, maxpos, x, y, len);
                     continue;
              case SCANCODE_END:
              case SCANCODE_KEYPAD1:
                     sound_send( SND_MENU);
                     pos = loopbar( maxpos, maxpos, x, y, len);
                     continue;
        } /* end switch */
 } /* end while( 1) */

}

int loopbar( int pos, int maxpos, int x, int y, int len)
{
 int i;

 for( i = 0; i < maxpos + 1; i++)
      draw_bar( x, y, len, i, LIN_BLACK);

 if( pos > maxpos) pos = 0;
 if( pos <      0) pos = maxpos;

 draw_bar( x, y, len, pos, LIN_RED);

 if( menu_function != NULL)
     menu_function( pos);

 return pos;
}

int check_world_magic( char *filename, int *stages)
{
 FILE *fp;
 struct world_header whdr;

 if( ( fp = fopen( filename, "r")) == NULL)
       return -1;

 if( fread( &whdr, sizeof( struct world_header), 1, fp) < 0)
     return -1;

 fclose( fp);

 *stages = whdr.number;

 if( strcmp( whdr.magic, WORLD_MAGIC))
     return -1;

 return 1;
}

void lin_chdir( char *dir)
{
 if( chdir( dir) == -1) {
     fprintf( stderr, _("\nCan't change to directory [%s] [%s]\n\n"), dir, strerror( errno));
     exit_game( 1);
 }
}

void lin_title_line( void)
{
 gl_fillbox( 0,  10, 319, 2, LIN_RED);
 gl_fillbox( 0, 186, 319, 2, LIN_RED);
}

int lin_inputstr( char *title, char *str, int qty)
{
 int pos, len, x;

 gl_clearscreen( 0);

 screen_center_text( title, 10);

 gl_fillbox( 0,  20, 319, 2, LIN_RED);
 gl_fillbox( 0,  45, 319, 2, LIN_RED);

 pos = 0;

 len = 8 * strlen( str) + 8;
 x = ( 320 - len) / 2;

rewrite_text:

 if( pos <       0) pos =       0;
 if( pos > qty - 1) pos = qty - 1;

 screen_center_text( str , 30);

 gl_fillbox( 0             , 40, 318, 2, LIN_BLACK);
 gl_fillbox( x + ( pos * 8), 40,   8, 2, LIN_RED);

 while( 1) {
        if( keyboard_update() == 0 && joy_moved() == 0)
            continue;

        switch( lin_scancode) {
                case SCANCODE_KEYPADENTER:
                case SCANCODE_ENTER:
                       return 1;
                case SCANCODE_ESCAPE:
                       return -1;
                case SCANCODE_CURSORUP:
                case SCANCODE_CURSORBLOCKUP:
                       str[ pos] = key_find_char( str[ pos], -1);
                       goto rewrite_text;
                case SCANCODE_CURSORLEFT:
                case SCANCODE_CURSORBLOCKLEFT:
                case SCANCODE_BACKSPACE:
                       pos--;
                       goto rewrite_text;
                case SCANCODE_CURSORDOWN:
                case SCANCODE_CURSORBLOCKDOWN:
                       str[ pos] = key_find_char( str[ pos], 1);
                       goto rewrite_text;
                case SCANCODE_CURSORRIGHT:
                case SCANCODE_CURSORBLOCKRIGHT:
                case SCANCODE_TAB:
                       pos++;
                       goto rewrite_text;
                case SCANCODE_HOME:
                       pos = 0;
                       goto rewrite_text;
                case SCANCODE_END:
                       pos = qty - 1;
                       goto rewrite_text;
                case LIN_KEY_NONE:
                       continue;
                default:
                       str[ pos] = keyttbl[ lin_scancode];
                       pos++;
                       goto rewrite_text;
        } /* end switch( lin_scancode) */

 } /* end while */

 /* Not reached, but compiler happy */
 return -1;
}

/*
 * I know that SVGALIB 1.3.x have a function called gl_printf()
 * but I found that lots of Linux distributions (the most used
 * ones) that still use the old SVGALIB 1.2.x, that doesn't have the
 * gl_printf() function, so I built one.
 */

void grf_printf( int x, int y, char *fmt, ...)
{
 char str[ 256];
 va_list ap;

 va_start( ap, fmt);

 vsprintf( str, fmt, ap);

 gl_write( x, y, str);

 va_end( ap);
}

int misc_wait( int ticks, int behavior)
{
 struct tms buf;
 clock_t w1, w2;

 w1 = times( &buf);
 w2 = w1;

 while( w2 < ticks + w1) {
        w2 = times( &buf);
        if( keyboard_update() == 0 && joy_moved() == 0)
            continue;

        switch( lin_scancode) {
                case SCANCODE_ENTER:
                case SCANCODE_KEYPADENTER:
                       if( behavior == WAIT_ESC_ENTER)
                           return 0;
                       else
                           return 1;
                case SCANCODE_Q:
                case SCANCODE_ESCAPE:
                       return 0;
        }
 } /* end while */

 return 1;
}

